# -*- coding: utf-8 -*-
"""
  Copyright 2022 Mitchell Isaac Parker

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

import pandas as pd
import streamlit as st

from ..constants.conf import sw1_color, sw2_color, sw1_name, sw2_name, loop_resid_dict
from ..constants.pharm import (
    sp2_name,
    none_pharm_name,
    other_pharm_name,
    pharm_color_dict,
)
from ..constants.prot import other_prot_name, none_prot_name, prot_color_dict
from ..functions.lig import lig_lst_dict
from ..functions.table import extract_int, lst_col
from ..functions.lst import str_to_lst
from ..functions.gui import (
    load_st_table,
    show_st_table,
    mask_st_table,
    show_st_structure,
    write_st_end,
    get_html_text,
)
from ..functions.col import (
    rename_col_dict,
    pdb_code_col,
    chainid_col,
    bio_lig_col,
    ion_lig_col,
    pharm_lig_col,
    chem_lig_col,
    mod_lig_col,
    mem_lig_col,
    gene_class_col,
    method_col,
    resolution_col,
    r_factor_col,
    space_col,
    mut_status_col,
    nuc_class_col,
    prot_class_col,
    pharm_class_col,
    match_class_col,
    pocket_class_col,
    interf_class_col,
    bound_prot_col,
    bound_prot_swiss_id_col,
    bound_prot_chainid_col,
    sw1_col,
    sw2_col,
    y32_col,
    y71_col,
    date_col,
)


def pdb_page():

    st.markdown("# Search PDB")

    st.markdown("---")

    df = load_st_table(__file__)

    st.sidebar.markdown("## PDB Selection")

    pdb_code = st.sidebar.selectbox(
        "Entry", [x.upper() for x in lst_col(df, pdb_code_col, unique=True)]
    )
    pdb_df = mask_st_table(df, {pdb_code_col: pdb_code.lower()})

    chainid = st.sidebar.selectbox("Chain", lst_col(pdb_df, chainid_col))
    chainid_df = mask_st_table(pdb_df, {chainid_col: chainid})

    st.markdown(
        f"### PDB: [{pdb_code}](https://www.rcsb.org/structure/{pdb_code}) ({chainid_df.at[0,gene_class_col]}) - Chain {chainid}"
    )

    left_col, right_col = st.columns(2)

    left_col.markdown("#### General Information")
    for col in [method_col, resolution_col, r_factor_col, space_col, date_col]:
        left_col.markdown(f"**{rename_col_dict[col]}:** {chainid_df.at[0,col]}")

    right_col.markdown("#### Molecular Annotations")

    annot_df = pd.DataFrame()
    for i, col in enumerate(
        [
            mut_status_col,
            nuc_class_col,
            prot_class_col,
            pocket_class_col,
            match_class_col,
            interf_class_col,
        ]
    ):
        annot_df.at[i, "Molecular Content"] = rename_col_dict[col]
        annot_df.at[i, "Annotation"] = chainid_df.at[0, col]

    show_st_table(annot_df, st_col=right_col)

    if chainid_df.at[0,bound_prot_col] != "None":
        for col in [bound_prot_col, bound_prot_swiss_id_col, bound_prot_chainid_col]:
            left_col.markdown(f"**{rename_col_dict[col]}:** {chainid_df.at[0,col]}")

    st_col_lst = st.columns(4)

    for i, col in enumerate([sw1_col, sw2_col, y32_col, y71_col]):

        if col in [sw1_col, y32_col]:
            color = sw1_color
        elif col in [sw2_col, y71_col]:
            color = sw2_color

        st_col_lst[i].markdown(f"#### {rename_col_dict[col]}")
        st_col_lst[i].markdown(get_html_text({chainid_df.at[0, col]:color}, 
        font_size="large"),unsafe_allow_html=True)

    st.markdown("---")

    left_check_col, right_check_col = st.columns(2)
    left_view_col, right_view_col = st.columns(2)

    left_check_col.markdown("#### Bound Ligands")

    lig_check_dict = dict()
    for col in [
        bio_lig_col,
        ion_lig_col,
        pharm_lig_col,
        chem_lig_col,
        mod_lig_col,
        mem_lig_col,
    ]:
        lig_lst = str_to_lst(chainid_df.at[0, col])
        if "None" not in lig_lst:
            lig_check_dict[col] = dict()
            for lig in lig_lst:
                lig_check_dict[col][lig] = left_check_col.checkbox(
                    f"{rename_col_dict[col]}: {lig}"
                )

    if len(lig_check_dict.keys()) == 0:
        left_check_col.write("No bound ligands.")

    right_check_col.markdown("#### Mutation Sites")

    mut_check_dict = dict()
    for mut in str_to_lst(chainid_df.at[0, mut_status_col]):
        if mut != "WT":
            mut_check_dict[mut] = right_check_col.checkbox(mut)

    if len(mut_check_dict.keys()) == 0:
        right_check_col.write("Not Mutated.")

    left_view_col.markdown("#### Viewer Settings")

    style_dict = {"Ribbon": "oval", "Trace": "trace"}

    cartoon_style = style_dict[
        left_view_col.radio("Cartoon Style", ["Ribbon", "Trace"])
    ]

    surf_trans = left_view_col.slider(
        "Surface Transparency", min_value=0.0, max_value=1.0
    )

    rotate_view = left_view_col.checkbox("Rotate Structure")


    style_lst = list()
    reslabel_lst = list()
    label_lst = list()

    opacity = 0
    if len(pdb_df) > 1:
        all_chains = left_view_col.checkbox("Show All Chains")

        if all_chains:
            opacity = 0.75

    style_lst.append(
        [
            {
                "chain": chainid,
                "invert": True,
            },
            {
                "cartoon": {
                    "color": "white",
                    "style": cartoon_style,
                    "thickness": 0.2,
                    "opacity": opacity,
                }
            },
        ]
    )

    for mut in list(mut_check_dict.keys()):
        style_lst.append(
            [
                {"chain": chainid, "resi": [extract_int(mut)], "atom": "CA"},
                {"sphere": {"color": "red", "radius": 0.8}},
            ]
        )
        if mut in list(mut_check_dict.keys()):
            if mut_check_dict[mut]:
                label_lst.append(
                    [
                        mut,
                        {
                            "backgroundColor": "lightgray",
                            "fontColor": "black",
                            "backgroundOpacity": 0.5,
                        },
                        {"chain": chainid, "resi": [extract_int(mut)], "atom": "CA"},
                    ]
                )

    for lig_col, lig_lst in lig_lst_dict.items():
        lig_lst = str_to_lst(chainid_df.at[0, lig_col])
        if "None" not in lig_lst:
            lig_style = "stick"
            lig_color = "whiteCarbon"
            lig_scheme = "colorscheme"
            lig_radius = 0.2
            if lig_col == ion_lig_col:
                lig_style = "sphere"
                lig_scheme = "color"
                lig_color = "chartreuse"
                lig_radius = 0.8
            for lig in lig_lst:
                if lig_col != pharm_lig_col:
                    if lig_col == mem_lig_col:
                        style_lst.append(
                            [
                                {
                                    "resn": [lig],
                                },
                                {
                                    lig_style: {
                                        lig_scheme: lig_color,
                                        "radius": lig_radius,
                                    }
                                },
                            ]
                        )

                    else:
                        style_lst.append(
                            [
                                {
                                    "chain": chainid,
                                    "resn": [lig],
                                },
                                {
                                    lig_style: {
                                        lig_scheme: lig_color,
                                        "radius": lig_radius,
                                    }
                                },
                            ]
                        )

                if lig_col in list(lig_check_dict.keys()):
                    if lig_check_dict[lig_col][lig]:
                        label_lst.append(
                            [
                                lig,
                                {
                                    "backgroundColor": "lightgray",
                                    "fontColor": "black",
                                    "backgroundOpacity": 0.5,
                                },
                                {
                                    "chain": chainid,
                                    "resn": lig,
                                },
                            ]
                        )

    pharm_class_lst = str_to_lst(chainid_df.at[0, pharm_class_col])

    if none_pharm_name not in pharm_class_lst:
        if len(pharm_class_lst) > 1:
            pharm_class = other_pharm_name
        else:
            pharm_class = pharm_class_lst[0]

        pharm_color = pharm_color_dict[pharm_class]

        pharm_lig_lst = str_to_lst(chainid_df.at[0, pharm_lig_col])

        for pharm_lig in pharm_lig_lst:
            style_lst.append(
                [
                    {
                        "chain": chainid,
                        "resn": [pharm_lig],
                        "elem": "C",
                    },
                    {"stick": {"color": pharm_color, "radius": 0.2}},
                ]
            )
            style_lst.append(
                [
                    {
                        "chain": chainid,
                        "resn": [pharm_lig],
                        "elem": ["N", "O", "H"],
                    },
                    {"stick": {"colorscheme": "lightgrayCarbon", "radius": 0.2}},
                ]
            )

            if pharm_class == sp2_name:
                style_lst.append(
                    [
                        {
                            "chain": chainid,
                            "resi": 12,
                        },
                        {"stick": {"colorscheme": "lightgrayCarbon", "radius": 0.2}},
                    ]
                )

    prot_class_lst = str_to_lst(chainid_df.at[0, prot_class_col])

    if none_prot_name not in prot_class_lst:
        if len(prot_class_lst) > 1:
            prot_class = other_prot_name
        else:
            prot_class = prot_class_lst[0]
        prot_color = prot_color_dict[prot_class]

        bound_chainid_lst = str_to_lst(chainid_df.at[0, bound_prot_chainid_col])

        for bound_prot_chainid in bound_chainid_lst:
            style_lst.append(
                [
                    {
                        "chain": bound_prot_chainid,
                    },
                    {
                        "cartoon": {
                            "style": cartoon_style,
                            "color": prot_color,
                            "thickness": 0.2,
                            "opacity": 1,
                        }
                    },
                ]
            )

            prot_label = prot_class
            if len(bound_chainid_lst) > 1:
                prot_label += f" - Chain {bound_prot_chainid}"

            label_lst.append(
                [
                    prot_label,
                    {
                        "backgroundColor": "lightgray",
                        "fontColor": "black",
                        "backgroundOpacity": 0.8,
                    },
                    {"chain": bound_prot_chainid},
                ]
            )

    surface_lst = [
        [
            {"opacity": surf_trans, "color": "white"},
            {"chain": chainid, "hetflag": False},
        ]
    ]

    for loop_name, loop_resids in loop_resid_dict.items():

        if loop_name == sw1_name:
            loop_color = sw1_color
            stick_resid = 32
        elif loop_name == sw2_name:
            loop_color = sw2_color
            stick_resid = 71

        surface_lst.append(
            [
                {"opacity": surf_trans, "color": loop_color},
                {"chain": chainid, "resi": loop_resids, "hetflag": False},
            ]
        )

        style_lst.append(
            [
                {
                    "chain": chainid,
                    "resi": loop_resids,
                },
                {
                    "cartoon": {
                        "style": cartoon_style,
                        "color": loop_color,
                        "thickness": 0.2,
                    }
                },
            ]
        )

        style_lst.append(
            [
                {
                    "chain": chainid,
                    "resi": stick_resid,
                    "elem": "C",
                },
                {"stick": {"color": loop_color, "radius": 0.2}},
            ]
        )

        style_lst.append(
            [
                {"chain": chainid, "resi": stick_resid, "elem": ["O", "N", "H"]},
                {"stick": {"colorscheme": "whiteCarbon", "radius": 0.2}},
            ]
        )

        reslabel_lst.append([{"chain": chainid, "resi": stick_resid}, {
                        "backgroundColor": "lightgray",
                        "fontColor": "black",
                        "backgroundOpacity": 0.8,
                    }])

    with right_view_col:
        show_st_structure(
            pdb_code,
            style_lst=style_lst,
            surface_lst=surface_lst,
            reslabel_lst=reslabel_lst,
            label_lst=label_lst,
            cartoon_style=cartoon_style,
            spin_on=rotate_view,
            zoom_dict={"chain": chainid},
            zoom=1.5,
            width=450,
            height=450,
        )

    write_st_end()


# -*- coding: utf-8 -*-
"""
  Copyright 2022 Mitchell Isaac Parker

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

import pandas as pd
import uuid
import re
from random import randint
import streamlit as st
import py3Dmol
from stmol import showmol

from .file import entry_table_file
from .table import mask_equal
from .path import (
    load_table,
    load_json,
    get_file_path,
    get_dir_path,
    get_neighbor_path,
    pages_str,
    data_str,
    functions_str,
)
from .col import rename_col_dict, date_col, nuc_class_col, match_class_col, prot_class_col, gene_class_col, interf_class_col, pocket_class_col
from .lst import type_lst

from ..constants.nuc import nuc_class_lst
from ..constants.pharm import match_class_lst, pocket_class_lst
from ..constants.prot import prot_class_lst
from ..constants.conf import sw1_name_lst, sw2_name_lst, y32_name_lst, y71_name_lst, y32_name, y71_name, sw1_name, sw2_name
from ..constants.gene import gene_class_lst
from ..constants.dimer import interf_class_lst

class_order_dict = {nuc_class_col: nuc_class_lst, match_class_col: match_class_lst, pocket_class_col: pocket_class_lst,
                    prot_class_col: prot_class_lst, sw1_name: sw1_name_lst, sw2_name: sw2_name_lst,
                    y32_name: y32_name_lst, y71_name: y71_name_lst,
                    gene_class_col: gene_class_lst, interf_class_col: interf_class_lst}


mitch_twitter = '<a href="https://twitter.com/Mitch_P?ref_src=twsrc%5Etfw" class="twitter-follow-button" data-show-count="false">Follow @Mitch_P</a><script async src="https://platform.twitter.com/widgets.js" charset="utf-8"></script>'
roland_twitter = '<a href="https://twitter.com/RolandDunbrack?ref_src=twsrc%5Etfw" class="twitter-follow-button" data-show-count="false">Follow @RolandDunbrack</a><script async src="https://platform.twitter.com/widgets.js" charset="utf-8"></script>'


def write_st_end():

    df = load_table(
        get_file_path(
            entry_table_file,
            dir_path=get_neighbor_path(__file__, functions_str, data_str),
        )
    )

    df[date_col] = pd.to_datetime(df[date_col])
    df[date_col] = df[date_col].dt.strftime("%Y-%m")

    st.markdown("---")
    st.markdown(
        "Developed and Maintained by Mitchell Parker, Bulat Faezov, and Roland Dunbrack"
    )
    st.markdown(
        "[Dunbrack Lab](https://dunbrack.fccc.edu/retro/) - [Fox Chase Cancer Center](https://www.foxchase.org)"
    )
    st.markdown(f"Most Recently Deposited Entry {df[date_col].max()}")
    st.markdown("Copyright (c) 2022 Mitchell Isaac Parker")

def reorder_st_cols(df, row, col):

    class_lst = list(class_order_dict.keys())

    if row in class_lst:
        row_order = [x for x in class_order_dict[row] + ["All"] if x in list(df.index.values)] 

    if col in class_lst:
        col_order = [x for x in class_order_dict[col] + ["All"] if x in list(df.columns)] 

    if row in class_lst and col in class_lst:
        df = df.loc[row_order, col_order]
    elif row not in class_lst and col in class_lst:
        df = df.loc[:, col_order]
    elif row in class_lst and col not in class_lst:
        df = df.loc[row_order, :]
    
    return df

def get_st_file_path(st_file):

    return get_file_path(
        f"{randint(0,3261994)}_{st_file.name}",
        dir_path=get_neighbor_path(__file__, functions_str, data_str),
    )


def save_st_file(st_file):

    st_file_path = get_st_file_path(st_file)
    with open(st_file_path, "wb") as file:
        file.write(st_file.getbuffer())
    return st_file_path


def get_html_text(text_color_dict, font_size="medium", font_weight="normal"):

    html_str = ""
    for text, color in text_color_dict.items():

        size = font_size
        if type(font_size) == dict:
            size = font_size[text]

        weight = font_weight
        if type(font_weight) == dict:
            weight = font_weight[text]

        html_str += f'<span style="font-family:sans-serif; font-size: {size}; font-weight: {weight}; color:{color};">{text}</span>'

    return html_str


def load_st_table(file_path, file_name=None, json_format=False):

    if file_name is None:
        file_name = entry_table_file

    file_path = get_file_path(
            file_name,
            dir_path=get_dir_path(
                dir_path=get_neighbor_path(file_path, pages_str, data_str)
            ))

    if json_format:
        return load_json(file_path)
    else:
        return load_table(file_path)
   
def mask_st_table(df, col_dict):

    mask_df = df.copy()

    for col in list(col_dict.keys()):
        val = type_lst(col_dict[col])
        if len(val) > 0:
            if "All" not in val:
                mask_df = mask_equal(mask_df, col, val)

    return mask_df


def rename_st_cols(df, col_lst=None):

    if col_lst is None:
        col_lst = [x for x in list(rename_col_dict.keys()) if x in list(df.columns)]

    return df.loc[:, col_lst].rename(columns=rename_col_dict)


def show_st_dataframe(df, st_col=None, hide_index=True):

    if hide_index:
        hide_dataframe_row_index = """
                <style>
                .row_heading.level0 {display:none}
                .blank {display:none}
                </style>
                """

        st.markdown(hide_dataframe_row_index, unsafe_allow_html=True)

    if st_col is None:
        st.dataframe(df)
    else:
        st_col.dataframe(df)


def show_st_table(df, st_col=None, hide_index=True):

    if hide_index:
        hide_table_row_index = """
                <style>
                tbody th {display:none}
                .blank {display:none}
                </style>
                """
        st.markdown(hide_table_row_index, unsafe_allow_html=True)

    if st_col is None:
        st.table(df)
    else:
        st_col.table(df)


def create_st_button(link_text, link_url, hover_color="#e78ac3", st_col=None):

    button_uuid = str(uuid.uuid4()).replace("-", "")
    button_id = re.sub("\d+", "", button_uuid)

    button_css = f"""
        <style>
            #{button_id} {{
                background-color: rgb(255, 255, 255);
                color: rgb(38, 39, 48);
                padding: 0.25em 0.38em;
                position: relative;
                text-decoration: none;
                border-radius: 4px;
                border-width: 1px;
                border-style: solid;
                border-color: rgb(230, 234, 241);
                border-image: initial;

            }}
            #{button_id}:hover {{
                border-color: {hover_color};
                color: {hover_color};
            }}
            #{button_id}:active {{
                box-shadow: none;
                background-color: {hover_color};
                color: white;
                }}
        </style> """

    html_str = f'<a href="{link_url}" target="_blank" id="{button_id}";>{link_text}</a><br></br>'

    if st_col is None:
        st.markdown(button_css + html_str, unsafe_allow_html=True)
    else:
        st_col.markdown(button_css + html_str, unsafe_allow_html=True)


def download_st_file(file_path, file_name, download_text, st_col=None):

    with open(file_path, "rb") as file:
        if st_col is None:
            st.download_button(download_text, file, file_name=file_name)
        else:
            st_col.download_button(download_text, file, file_name=file_name)


def encode_st_df(df):

    return df.to_csv(sep="\t", index=False).encode("utf-8")


def download_st_df(df, file_name, download_text, st_col=None):

    if st_col is None:
        st.download_button(
            label=download_text,
            data=encode_st_df(df),
            file_name=file_name,
        )
    else:
        st_col.download_button(
            label=download_text,
            data=encode_st_df(df),
            file_name=file_name,
        )


def show_st_structure(
    pdb_code,
    style_lst=None,
    label_lst=None,
    reslabel_lst=None,
    zoom_dict=None,
    surface_lst=None,
    cartoon_style="trace",
    cartoon_radius=0.2,
    cartoon_color="lightgray",
    zoom=1,
    spin_on=False,
    width=900,
    height=600,
):

    view = py3Dmol.view(query=f"pdb:{pdb_code.lower()}", width=width, height=height)

    view.setStyle(
        {
            "cartoon": {
                "style": cartoon_style,
                "color": cartoon_color,
                "thickness": cartoon_radius,
            }
        }
    )

    if surface_lst is not None:
        for surface in surface_lst:
            view.addSurface(py3Dmol.VDW, surface[0], surface[1])

    if style_lst is not None:
        for style in style_lst:
            view.addStyle(
                style[0],
                style[1],
            )

    if label_lst is not None:
        for label in label_lst:
            view.addLabel(label[0], label[1], label[2])

    if reslabel_lst is not None:
        for reslabel in reslabel_lst:
            view.addResLabels(reslabel[0], reslabel[1])


    if zoom_dict is None:
        view.zoomTo()
    else:
        view.zoomTo(zoom_dict)

    view.spin(spin_on)

    view.zoom(zoom)
    showmol(view, height=height, width=width)



# -*- coding: utf-8 -*-
"""
  Copyright 2022 Mitchell Isaac Parker

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

import pandas as pd
import numpy as np

from ..functions.chem import draw_lig_plot, get_lig_mcs, get_lig_smiles, get_lig_simi, is_lig_match
from ..functions.table import mask_equal, lst_col, mask_unequal, str_to_dict
from ..functions.col import (pocket_class_col, pdb_id_col, pdb_code_col, 
bound_lig_cont_col, chainid_col, pharm_lig_smiles_col, pharm_class_col,
pocket_lig_col, pharm_lig_col, pocket_score_col, match_class_col,
pocket_volume_col, pharm_lig_col, rename_col_dict)
from ..functions.file import pocket_table_file
from ..constants.pharm import sp2_name, sp12_name, bound_name, pharm_color_dict
from ..constants.conf import loop_resid_dict, loop_color_dict, sw1_name, sw2_name, y32_name, y71_name
from ..functions.gui import load_st_table, write_st_end, show_st_structure, show_st_table, get_html_text
from ..functions.lst import calc_simpson, res_to_lst,lst_unique, str_to_lst

import streamlit as st

sp12_bound_name = f"{sp12_name}-{bound_name}"
sp2_bound_name = f"{sp2_name}-{bound_name}"

def inhibitor_page():
    
    st.markdown("# Compare Inhibitors")

    st.markdown("---")

    df = load_st_table(__file__)
    pocket_df = load_st_table(__file__, file_name=pocket_table_file)

    pocket_df = mask_unequal(pocket_df, pocket_lig_col, "STP")

    df = mask_equal(df, pocket_class_col, [sp12_bound_name, sp2_bound_name])
    df = mask_equal(df, pdb_id_col, lst_col(pocket_df, pdb_id_col))

    left_query_col, right_query_col = st.columns(2)

    pocket_class = left_query_col.radio(rename_col_dict[pocket_class_col], [sp12_bound_name, sp2_bound_name])
    pharm_class = pocket_class.split('-')[0]

    pharm_df = mask_equal(df, pharm_class_col, pharm_class)

    for col in [match_class_col]:

        mask_lst = right_query_col.multiselect(rename_col_dict[col], 
                                    lst_col(pharm_df, col, unique=True))

        if len(mask_lst) > 0:
            pharm_df = mask_equal(df, col, mask_lst)


    with st.expander("Additional Site Query", expanded=False):

        cont_lst = st.multiselect("Residue Contacts", res_to_lst("1-189"))

        if len(cont_lst) > 0:

            cont_simi = st.slider("Residue Contact Similarity (Simpson)",  min_value=0.0, max_value=1.0, value=0.4)

            st.write("Querying for Sites")
            site_bar = st.progress(0)
            site_index_lst = list()
            for i, index in enumerate(list(pharm_df.index.values)):
                site_cont_dict = str_to_dict(pharm_df.at[index, bound_lig_cont_col], return_int=True)
                site_cont_lst = site_cont_dict[pharm_df.at[index, pharm_lig_col]]
                if calc_simpson(site_cont_lst, cont_lst) >= cont_simi:
                    site_index_lst.append(index)

                site_bar.progress((i + 1)/len(pharm_df))

            pharm_df = pharm_df.loc[site_index_lst, :]

    with st.expander("Additional Chemistry Query", expanded=False):

        smiles_str = st.text_input("SMILES Strings (Comma-Separated)")

        st.markdown("**Tip.** Use [PubChem Draw Tool](https://pubchem.ncbi.nlm.nih.gov/#draw=true) to design and edit SMILES strings.")

        if len(smiles_str) > 0:

            smiles_lst = str_to_lst(smiles_str)

            exact_name = 'Exact'
            simi_name = 'Similarity'

            match_type = st.radio("Substructure Match", [exact_name, simi_name])

            if match_type == exact_name:
                min_match = st.number_input("Minimum Matches", min_value=1, max_value=len(smiles_lst), value=1)

            elif match_type == simi_name:
                mean_name = "Mean"
                max_name = "Max"
                min_name = "Min"

                min_simi = st.slider("2D Fingerprint Similarity (Tonimoto)", min_value=0.0, max_value=1.0, value=0.1)

                if len(smiles_lst) > 1:
                    simi_method = st.radio("Similarity Method", [mean_name, max_name, min_name])

            st.write("Querying for Chemistries")
            chem_bar = st.progress(0.0)
            chem_index_lst = list()
            for i, index in enumerate(list(pharm_df.index.values)):
                chem_smiles_dict = str_to_dict(pharm_df.at[index, pharm_lig_smiles_col], return_str=True)
                chem_smiles= chem_smiles_dict[pharm_df.at[index, pharm_lig_col]][0]
                if match_type == exact_name:
                    if is_lig_match(lig=chem_smiles,matches=smiles_lst) >= min_match:
                        chem_index_lst.append(index)
                elif match_type == simi_name:
                    lig_simi_lst = [get_lig_simi(chem_smiles, x) for x in smiles_lst]
                    if len(smiles_lst) == 1:
                        lig_simi = lig_simi_lst[0]
                    else:
                        if simi_method == mean_name:
                            lig_simi = np.mean(lig_simi_lst)
                        elif simi_method == max_name:
                            lig_simi = np.max(lig_simi_lst)
                        elif simi_method == min_name:
                            lig_simi = np.min(lig_simi_lst)

                    if lig_simi >= min_simi:
                        chem_index_lst.append(index)

                chem_bar.progress((i + 1)/len(pharm_df))

            pharm_df = pharm_df.loc[chem_index_lst, :]

    for col in [pharm_lig_col]:

        mask_lst = st.multiselect(rename_col_dict[col], 
                                    lst_col(pharm_df, col, unique=True))

        if len(mask_lst) > 0:
            pharm_df = mask_equal(df, col, mask_lst)

    st.info(f"Total of {len(pharm_df)} inhibitor-bound structures based on provided query.")

    if len(pharm_df) < 2:
        st.warning("Insufficient Number of Structures Based On Selected Query")
    else:
        with st.expander(f"{pocket_class}: One-to-One Comparison", expanded=True):

            st.markdown(f"#### {pocket_class}: One-to-One Comparison")

            left_pdb_col, right_pdb_col = st.columns(2)

            pdb_id_lst = [x.upper() for x in lst_col(pharm_df, pdb_id_col)]
            
            pdb_id_1 = left_pdb_col.selectbox("PDB ID (Left)", pdb_id_lst)

            pdb_id_2 = right_pdb_col.selectbox("PDB ID (Right)", [x for x in pdb_id_lst if x != pdb_id_1])

            pdb_code_1 = pdb_id_1[:4].lower()
            chainid_1 = pdb_id_1[4:5]

            pdb_code_2 = pdb_id_2[:4].lower()
            chainid_2 = pdb_id_2[4:5]

            pdb_id_1 = f"{pdb_code_1}{chainid_1}"
            pdb_id_2 = f"{pdb_code_2}{chainid_2}"

            pharm_1_df = mask_equal(pocket_df, pdb_id_col, pdb_id_1)
            pharm_2_df = mask_equal(pocket_df, pdb_id_col, pdb_id_2)

            pocket_1_df = mask_equal(pocket_df, pdb_id_col, pdb_id_1)
            pocket_2_df = mask_equal(pocket_df, pdb_id_col, pdb_id_2)

            view_dict = {pdb_id_1: [left_pdb_col, pharm_1_df, pocket_1_df], pdb_id_2: [right_pdb_col, pharm_2_df, pocket_2_df]}

            st.sidebar.markdown("## Viewer Settings")

            show_cont = st.sidebar.checkbox("Label Contacts",value=True)

            style_dict = {"Ribbon": "oval", "Trace": "trace"}

            scheme_dict = {"Standard": False, "Amino Acid": True}
            amino_scheme = scheme_dict[
                st.sidebar.radio("Residue Colors", ["Standard", "Amino Acid"])
            ]

            cartoon_style = style_dict[
                st.sidebar.radio("Cartoon Style", ["Ribbon", "Trace"])
            ]

            cartoon_trans = st.sidebar.slider(
                "Cartoon Transparency", min_value=0.0, max_value=1.0, value=0.5
            )

            surf_trans = st.sidebar.slider(
                "Surface Transparency", min_value=0.0, max_value=1.0, value=0.0
            )

            for pdb_id in list(view_dict.keys()):
                view_col = view_dict[pdb_id][0]
                pharm_pdb_df = view_dict[pdb_id][1]
                pocket_pdb_df = view_dict[pdb_id][1]

                pdb_code = pharm_pdb_df[pdb_code_col].iloc[0]
                chainid = pharm_pdb_df[chainid_col].iloc[0]
                lig = pharm_pdb_df[pharm_lig_col].iloc[0]
                match_class = pharm_pdb_df[match_class_col].iloc[0]
                smiles_dict = str_to_dict(pharm_pdb_df[pharm_lig_smiles_col].iloc[0], return_str=True)
                cont_dict = str_to_dict(pharm_pdb_df[bound_lig_cont_col].iloc[0], return_int=True)

                smiles = smiles_dict[lig][0]

                view_col.markdown(f"**{rename_col_dict[pharm_lig_col]}:** [{lig}](https://www.rcsb.org/ligand/{lig}) ({match_class})")
                view_col.markdown(f"**SMILES:** {smiles}")
                
                for col in [pocket_score_col, pocket_volume_col]:
                    view_col.markdown(f"**{rename_col_dict[col]}:** {round(float(pocket_pdb_df[col].max()),2)}")

                style_lst = list()
                reslabel_lst = list()
                surface_lst = list()

                style_lst.append(
                    [
                        {
                            "chain": chainid,
                            "invert": True,
                        },
                        {
                            "cartoon": {
                                "color": "white",
                                "style": cartoon_style,
                                "thickness": 0.2,
                                "opacity": 0,
                            }
                        },
                    ]
                )

                style_lst.append(
                    [
                        {
                            "chain": chainid,
                        },
                        {
                            "cartoon": {
                                "color": "white",
                                "style": cartoon_style,
                                "thickness": 0.2,
                                "opacity": cartoon_trans,
                            }
                        },
                    ]
                )  

                style_lst.append(
                        [
                            {
                                "chain": chainid,
                                "resn": [lig],
                                "elem": "C",
                            },
                            {"stick": {"color": pharm_color_dict[pharm_class], "radius": 0.2}},
                        ]
                    )

                style_lst.append(
                    [
                        {
                            "chain": chainid,
                            "resn": [lig],
                            "elem": ["N", "O", "H"],
                        },
                        {"stick": {"colorscheme": "lightgrayCarbon", "radius": 0.2}},
                    ]
                )

                if sp2_name in pocket_class:
                    style_lst.append(
                        [
                            {
                                "chain": chainid,
                                "resi": 12,
                            },
                            {"stick": {"colorscheme": "lightgrayCarbon", "radius": 0.2}},
                        ]
                    )

                surface_lst = [
                [
                    {"opacity": surf_trans, "color": "white"},
                    {"chain": chainid, "hetflag": False},
                ]
            ]

                sw1_resid_lst = res_to_lst(loop_resid_dict[sw1_name])
                sw2_resid_lst = res_to_lst(loop_resid_dict[sw2_name])

                if amino_scheme:
                    style_lst.append(
                        [
                            {"chain": chainid,
                                "resi": cont_dict[lig],
                                "elem": "C",},
                            {"stick": {"colorscheme": 'amino', "radius": 0.2}},
                        ]
                    ) 
                    style_lst.append(
                            [
                                {"chain": chainid, "resi": cont_dict[lig], "elem": ["O", "N", "H"]},
                                {"stick": {"colorscheme": "whiteCarbon", "radius": 0.2}},
                            ]
                        )

                else:
                    for resid in cont_dict[lig]:

                        resid_color = "white"
                        if int(resid) in sw1_resid_lst:
                            resid_color = loop_color_dict[sw1_name]
                        if int(resid) in sw2_resid_lst:
                            resid_color = loop_color_dict[sw2_name]


                        style_lst.append(
                        [
                            {
                                "chain": chainid,
                                "resi": [resid],
                                "elem": "C",
                            },
                            {"stick": {"color": resid_color, "radius": 0.2}},
                        ]
                    )

                        style_lst.append(
                            [
                                {"chain": chainid, "resi": [resid], "elem": ["O", "N", "H"]},
                                {"stick": {"colorscheme": "whiteCarbon", "radius": 0.2}},
                            ]
                        )

                if show_cont:

                    reslabel_lst.append(
                            [
                                {"chain": chainid,
                                "resi": cont_dict[lig]},
                                {
                                    "backgroundColor": "lightgray",
                                    "fontColor": "black",
                                    "backgroundOpacity": 0.5,
                                },
                            ]
                        )

                for loop_name, loop_resids in loop_resid_dict.items():

                    loop_color = loop_color_dict[loop_name]

                    surface_lst.append(
                    [
                        {"opacity": surf_trans, "color": loop_color},
                        {"chain": chainid, "resi": loop_resids, "hetflag": False},
                    ]
                )

                    style_lst.append(
                        [
                            {
                                "chain": chainid,
                                "resi": [loop_resids],
                            },
                            {
                                "cartoon": {
                                    "style": cartoon_style,
                                    "color": loop_color,
                                    "thickness": 0.2,
                                    "opacity": cartoon_trans,
                                }
                            },
                        ]
                    )

                    

                with view_col:
                    show_st_structure(
                        pdb_code,
                        style_lst=style_lst,
                        reslabel_lst=reslabel_lst,
                        surface_lst=surface_lst,
                        cartoon_style=cartoon_style,
                        zoom_dict={"chain": chainid, "resn": lig},
                        zoom=0.7,
                        width=450,
                        height=300,
                    )

                label_size = "medium"

                for col in [sw1_name, sw2_name, y32_name, y71_name]:

                    label_str = get_html_text({f"{rename_col_dict[col]}: ": "#31333F"}, font_weight='bold', font_size=label_size)

                    if col in [y32_name, sw1_name]:
                        label_color = loop_color_dict[sw1_name]
                    elif col in [y71_name, sw2_name]:
                        label_color = loop_color_dict[sw2_name]

                    label_str += get_html_text({pharm_pdb_df[col].iloc[0]: label_color}, font_size=label_size)

                    view_col.markdown(label_str, unsafe_allow_html=True)

            
            if amino_scheme:

                st.sidebar.markdown("---")

                st.sidebar.markdown("## Amino Acid Colors")

                aa_type_dict = {'Aromatic':['PHE', 'TYR', 'TRP'],
                'Acidic':['ASP','GLU'],
                'Basic':['LYS', 'ARG','HIS'],
                'Nonpolar':['ILE', 'VAL', 'LEU', 'MET','PRO', 'GLY', 'ALA'],
                'Polar':['ASN','GLN','SER','THR', 'CYS']}

                aa_color_dict = {'ASP':'#E60A0A','GLU':'#E60A0A','CYS':'#E6E600','MET':'#E6E600',
                                'LYS':'#145AFF','ARG':'#145AFF','SER':'#FA9600','THR':'#FA9600',
                                'PHE':'#3232AA','TYR':'#3232AA','ASN':'#00DCDC','GLN':'#00DCDC',
                                'GLY':'#C8C8C8','LEU':'#0F820F','VAL':'#0F820F','ILE':'#0F820F',
                                'ALA':'#C8C8C8','TRP':'#B45AB4','HIS':'#8282D2','PRO':'#DC9682'}


                aa_font_size = "medium"

                for aa_type, aa_lst in aa_type_dict.items():

                    aa_str = get_html_text({f"{aa_type}: ": "#31333F"}, font_weight='bold', font_size=aa_font_size)

                    for i, aa_name in enumerate(aa_lst): 

                        aa_str += get_html_text({aa_name:aa_color_dict[aa_name]}, font_size=aa_font_size)

                        if i != len(aa_lst) - 1:
                            aa_str += get_html_text({", ":"#31333F"}, font_size=aa_font_size)
        
                    st.sidebar.markdown(aa_str, unsafe_allow_html=True)


            st.markdown("---")

            left_chem_col, right_chem_col = st.columns(2)

            lig_1 = pharm_1_df[pharm_lig_col].iloc[0]
            lig_2 = pharm_2_df[pharm_lig_col].iloc[0]

            smiles_dict_1 = str_to_dict(pharm_1_df[pharm_lig_smiles_col].iloc[0])
            smiles_dict_2 = str_to_dict(pharm_2_df[pharm_lig_smiles_col].iloc[0])
            
            smiles_1 = smiles_dict_1[lig_1][0]
            smiles_2 = smiles_dict_2[lig_2][0]

            right_chem_col.markdown("#### Chemistry Comparison")

            mcs = get_lig_mcs([smiles_1, smiles_2])

            right_chem_col.markdown(f"**2D Fingerprint Similarity (Tanimoto):** {round(get_lig_simi(smiles_1, smiles_2),2)}")

            right_chem_col.markdown(f"**Maximum Common Substructure (MCS):** {get_lig_smiles(mcs)}")

            if not right_chem_col.checkbox(f"Show MCS",value=True):
                mcs = None
            
            right_chem_col.image(draw_lig_plot([smiles_1, smiles_2], 
            lig_labels=[f"{pdb_id_1.upper()} ({lig_1})", f"{pdb_id_2.upper()} ({lig_2})"], font_size=16,
            plot_height=4, plot_width=3, highlight_querys=mcs, color_palette=[pharm_color_dict[pharm_class]]))

            cont_dict_1 = str_to_dict(pharm_1_df[bound_lig_cont_col].iloc[0])
            cont_dict_2 = str_to_dict(pharm_2_df[bound_lig_cont_col].iloc[0])

            cont_lst_1 = cont_dict_1[lig_1]
            cont_lst_2 = cont_dict_2[lig_2]
            
            cont_df = pd.DataFrame()

            left_chem_col.markdown("#### Site Comparison")

            left_chem_col.markdown(f"**Residue Contact Similarity (Simpson):** {round(calc_simpson(cont_lst_1, cont_lst_2),2)}")

            cont_col_1 = f"{pdb_id_1.upper()} ({lig_1})"
            cont_col_2 = f"{pdb_id_2.upper()} ({lig_2})"

            for resid in sorted(lst_unique(cont_lst_1 + cont_lst_2,return_int=True)):

                cont_1 = ''
                cont_2 = ''

                if str(resid) in cont_lst_1:
                    cont_1 = "✓"
                if str(resid) in cont_lst_2:
                    cont_2 = "✓"

                cont_df.at[resid, cont_col_1] = cont_1
                cont_df.at[resid, cont_col_2] = cont_2

            cont_df = cont_df.rename_axis('Residue Contact').reset_index()

            show_st_table(cont_df, st_col=left_chem_col)

        with st.expander(f"{pocket_class}: Multiple-to-Multiple Comparison", expanded=True):

            st.markdown(f"#### {pocket_class}: Multiple-to-Multiple Comparison")
            st.markdown(f"#### IN PROGRESS")


    write_st_end()


# -*- coding: utf-8 -*-
"""
  Copyright 2022 Mitchell Isaac Parker

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""
import os
import pandas as pd

from ..scripts.build_dih_table import build_dih_table
from ..scripts.build_dist_table import build_dist_table
from ..scripts.make_facet_plot import make_facet_plot, grid_hex
from ..scripts.write_pymol_script import write_pymol_script
from ..scripts.prep_pocket import pocket_bound_name, pocket_unbound_name

from ..constants.dimer import dimer_name
from ..constants.prot import (
    prot_class_lst,
    prot_color_dict,
    none_prot_name,
    gap_name,
    gef_cdc_name,
    binder_name,
    nano_name,
    other_prot_name,
    mult_prot_name
)
from ..constants.pharm import (
    sp12_name,
    sp2_name,
    other_pharm_name,
    pharm_color_dict,
    pharm_class_lst,
    none_pharm_name,
    other_pharm_name,
    mult_pharm_name
)
from ..constants.gene import hras_name
from ..constants.nuc import nuc_class_lst, nuc_color_dict, gtp_atomids, gtp_name
from ..constants.conf import (
    loop_resid_dict,
    conf_color_dict,
    conf_nuc_color_dict,
    outlier_name,
    disorder_name,
    sw1_name,
    sw2_name,
    sw1_resids,
    sw2_resids,
    sw1_color,
    sw2_color,
    y32_name,
    q61_name,
    y71_name,
    resid_color_dict,
    sw1_gdp_out_off_name,
    sw1_gtp_in_on_name,
    sw1_nf_out_gef_name,
    sw2_gtp_in_r_name,
    sw2_gtp_in_sp12a_name,
    sw2_gdp_in_sp12_name,
    sw2_gdp_out_binder_name,
    sw2_gdp_out_sp2a_name,
    sw2_gdp_out_sp2b_name,
    sw2_gtp_in_sp12b_name,
    sw2_gtp_out_t_name,
    sw2_nf_out_gef_name,
    hb_color_dict
)
from ..constants.pocket import pocket_site_lst, pocket_color_dict
from ..constants.pml import (
    sup_resids,
    show_resids,
    sup_pdb_code,
    sup_chainid,
    mono_view,
    y32_view,
    gap_view,
    gef_view,
    mut_view,
    sw1_mono_view,
    sw2_mono_view,
    sw1_prot_view,
    sw2_prot_view,
)
from ..functions.stat import calc_rr
from ..functions.plot import make_venn_plot, make_stacked_barplot
from ..functions.color import change_hex_alpha, get_palette_hex_lst, gray_hex, blue_hex, orange_hex, green_hex, purple_hex, pink_hex, cyan_hex
from ..functions.file import (
    entry_table_file,
    dist_table_file,
    dih_table_file,
    pocket_table_file,
    interf_table_file,
    dih_json_file,
    pymol_pml_file,
    plot_img_file,
    venn_img_file,
    stat_table_file,
    sum_table_file
)
from ..functions.coord import wmhb_name, hb_name
from ..functions.path import (
    get_file_path,
    load_table,
    load_json,
    save_table,
    get_core_path,
    rascore_str,
    plot_str,
)
from ..functions.col import (
    rama_col,
    nuc_class_col,
    phi_col,
    psi_col,
    cluster_col,
    pdb_id_col,
    core_path_col,
    bio_lig_col,
    atom_dist_col,
    hb_status_col,
    outlier_col,
    gene_class_col,
    mut_class_col,
    chi1_col,
    chi2_col,
    pocket_class_col,
    pocket_type_col,
    pocket_site_col,
    pocket_status_col,
    pocket_volume_col,
    pocket_score_col,
    pocket_path_col,
    pharm_class_col,
    rotamer_col,
    match_class_col,
    prot_class_col,
    interf_path_col,
    bound_interf_chainid_col,
    bound_prot_cont_col,
    bound_lig_cont_col,
    sig_col,
    risk_ratio_col,
    interf_class_col
)

from ..functions.table import (
    mask_equal,
    mask_greater,
    mask_unequal,
    get_col_most_common,
    lst_col,
    build_col_count_dict,
    make_dict,
)
from ..functions.lst import res_to_lst, str_to_lst, type_lst, lst_to_str


def plot_rama(df, dih_dict, out_path, num_cpu=1):

    dih_table_path = get_file_path(
        dih_table_file, dir_str=rama_col, dir_path=out_path, pre_str=False
    )

    g_resids = "2-166"

    # build_dih_table(
    #     df, dih_dict, dih_table_path=dih_table_path, bb_resids=g_resids, num_cpu=num_cpu
    # )

    # dih_df = load_table(dih_table_path)

    # make_facet_plot(
    #     dih_df,
    #     get_file_path(f"{rama_col}_{plot_img_file}", dir_str=rama_col, dir_path=out_path, pre_str=False),
    #     x_col=phi_col,
    #     y_col=psi_col,
    #     x_str="φ",
    #     y_str="ψ",
    #     hue_col=nuc_class_col,
    #     hue_palette=nuc_color_dict,
    #     col_wrap=11,
    #     plot_width=5,
    #     plot_height=7,
    #     darken_lst=[
    #         res_to_lst(sup_resids),
    #     ],
    #     darken_palette=[grid_hex],
    #     rename_col=res_to_lst(g_resids),
    #     show_legend=True,
    #     hue_count=True,
    #     legend_pad=5,
    # )

    major_name = "Major"
    minor_name = "Minor"

    color_dict = dict()

    for nuc_class in nuc_class_lst:
        nuc_color = nuc_color_dict[nuc_class]
        color_dict[f"{nuc_class}-{major_name}"] = nuc_color
        color_dict[f"{nuc_class}-{minor_name}"] = change_hex_alpha(nuc_color, 0.5)

    for loop_name, loop_resids in loop_resid_dict.items():

        dih_table_path = get_file_path(
            dih_table_file, dir_str=loop_name, dir_path=out_path
        )

        # build_dih_table(
        #     df,
        #     dih_dict,
        #     dih_table_path=dih_table_path,
        #     bb_resids=loop_resids,
        #     num_cpu=num_cpu,
        # )

        dih_df = load_table(dih_table_path)

        rama_dict = dict()

        for cluster in lst_col(dih_df, loop_name, unique=True):
            if cluster not in [outlier_name, disorder_name]:
                rama_dict[cluster] = [
                    x
                    for x in get_col_most_common(
                        mask_equal(dih_df, loop_name, cluster), rama_col
                    )
                    if "-" not in x
                ][0]

        for index in list(dih_df.index.values):
            cluster = dih_df.at[index, loop_name]
            rama_status = major_name
            if cluster not in [outlier_name, disorder_name]:
                if dih_df.at[index, rama_col] != rama_dict[cluster]:
                    rama_status = minor_name
            dih_df.at[index, nuc_class_col] += "-"
            dih_df.at[index, nuc_class_col] += rama_status

        make_facet_plot(
            dih_df,
            get_file_path(
                f"{rama_col}_{plot_img_file}", dir_str=loop_name, dir_path=out_path
            ),
            x_col=phi_col,
            y_col=psi_col,
            x_str="φ",
            y_str="ψ",
            hue_col=nuc_class_col,
            row_col=loop_name,
            row_order=list(conf_nuc_color_dict[loop_name].keys()),
            rename_col=res_to_lst(loop_resids),
            hue_palette=color_dict,
            row_palette=conf_nuc_color_dict[loop_name],
            plot_width=5,
            show_legend=False,
            marker_size=1,
        )


def plot_pymol(df, interf_df, sup_core_path, out_path):

    dark_hex_lst = get_palette_hex_lst('Dark2')

    sw1_pymol_name_dict = {'1q21A':"GDP-bound","1bkdR":"Nucleotide-Free","4eflA":"State 1","5p21A":"State 2"}
    sw2_pymol_name_dict = {"5b2zA":"T state","3k8yA":"R State"}

    sw1_pymol_color_dict = {"GDP-bound":dark_hex_lst[0],"Nucleotide-Free":dark_hex_lst[1],"State 1":dark_hex_lst[2],"State 2":dark_hex_lst[3]}
    sw2_pymol_color_dict = {"T state":dark_hex_lst[4],"R State":dark_hex_lst[5]}

    sw1_df = mask_equal(df, pdb_id_col, list(sw1_pymol_name_dict.keys()))
    sw2_df = mask_equal(df, pdb_id_col, list(sw2_pymol_name_dict.keys()))

    sw1_df[sw1_name] = sw1_df[pdb_id_col].map(sw1_pymol_name_dict)
    sw2_df[sw2_name] = sw2_df[pdb_id_col].map(sw2_pymol_name_dict)

    write_pymol_script(
        sw1_df,
        get_file_path(f"{sw1_name}_{pymol_pml_file}", dir_path=out_path),
        stick_resids=[32],
        loop_resids=[sw1_resids],
        group_col=sw1_name,
        color_palette=sw1_pymol_color_dict,
        color_group=True,
        show_bio=True,
        show_ion=True,
        style_ribbon=False,
        sup_resids=sup_resids,
        sup_coord_path=sup_core_path,
        sup_chainid=sup_chainid,
        set_view=mono_view,
        show_resids=show_resids,
    )

    write_pymol_script(
        sw2_df,
        get_file_path(f"{sw2_name}_{pymol_pml_file}", dir_path=out_path),
        stick_resids=[71],
        loop_resids=[sw2_resids],
        group_col=sw2_name,
        color_palette=sw2_pymol_color_dict,
        color_group=True,
        show_bio=True,
        show_ion=True,
        style_ribbon=False,
        sup_resids=sup_resids,
        sup_coord_path=sup_core_path,
        sup_chainid=sup_chainid,
        set_view=mono_view,
        show_resids=show_resids,
    )

    for resid_name in [y32_name, y71_name]:

        if resid_name == y32_name:
            loop_resids = sw1_resids
            stick_resids = [32]
        elif resid_name == y71_name:
            loop_resids = sw2_resids
            stick_resids = [71]

        for nuc_class in nuc_class_lst:

            write_pymol_script(
                mask_equal(mask_unequal(df,resid_name,disorder_name),nuc_class_col,nuc_class),
                get_file_path(f"{nuc_class}_{pymol_pml_file}", dir_str=resid_name, dir_path=out_path),
                group_col=resid_name,
                stick_resids=stick_resids,
                style_ribbon=True,
                thick_bb=False,
                show_bio=True,
                color_palette=resid_color_dict[resid_name],
                sup_resids=sup_resids,
                sup_coord_path=sup_core_path,
                sup_chainid=sup_chainid,
                set_view=mono_view,
                show_resids=show_resids,
            )


    for loop_name, loop_resids in loop_resid_dict.items():

        if loop_name == sw1_name:
            stick_resids = [32]
        elif loop_name == sw2_name:
            stick_resids = [71]

        write_pymol_script(
            mask_unequal(df,loop_name,[outlier_name,disorder_name]),
            get_file_path(pymol_pml_file, dir_str=loop_name, dir_path=out_path),
            group_col=loop_name,
            stick_resids=stick_resids,
            loop_resids=loop_resids,
            style_ribbon=True,
            thick_bb=False,
            show_bio=True,
            color_palette=conf_color_dict[loop_name],
            sup_resids=sup_resids,
            sup_coord_path=sup_core_path,
            sup_chainid=sup_chainid,
            set_view=mono_view,
            show_resids=show_resids,
        )

    for loop_name, loop_resids in loop_resid_dict.items():
        for prot_class in [x for x in prot_class_lst if x != none_prot_name]:

            prot_df = mask_equal(df, prot_class_col, prot_class)

            if loop_name == sw1_name:
                stick_resids = [32]
                if prot_class == gap_name:
                    set_view = gap_view
                elif gef_cdc_name in prot_class:
                    set_view = gef_view
                elif binder_name in prot_class:
                    set_view = sw1_mono_view
                else:
                    set_view = sw1_prot_view
            elif loop_name == sw2_name:
                stick_resids = [71]
                set_view = sw2_prot_view
                if binder_name in prot_class:
                    set_view = sw2_mono_view

            write_pymol_script(
                prot_df,
                get_file_path(
                    f"{loop_name}_{prot_class}_{pymol_pml_file}",
                    dir_str=prot_class_col,
                    dir_path=out_path,
                    pre_str=False
                ),
                group_col=loop_name,
                stick_resids=stick_resids,
                loop_resids=loop_resids,
                style_ribbon=True,
                thick_bb=False,
                show_bio=True,
                show_prot=prot_color_dict[prot_class.split(".")[0]],
                color_palette=conf_color_dict[loop_name],
                sup_resids=sup_resids,
                sup_coord_path=sup_core_path,
                sup_chainid=sup_chainid,
                set_view=set_view,
                show_resids=show_resids,
            )

    for loop_name, loop_resids in loop_resid_dict.items():
        for pharm_class in [x for x in pharm_class_lst if x != none_pharm_name]:

            pharm_df = mask_equal(df, pharm_class_col, pharm_class)

            if loop_name == sw1_name:
                stick_resids = [32]
                set_view = sw1_mono_view
            elif loop_name == sw2_name:
                stick_resids = [71]
                set_view = sw2_mono_view

            if sp2_name in pharm_class:
                stick_resids += [12]

            write_pymol_script(
                pharm_df,
                get_file_path(
                    f"{loop_name}_{pharm_class}_{pymol_pml_file}",
                    dir_str=pharm_class_col,
                    dir_path=out_path,
                ),
                group_col=loop_name,
                stick_resids=stick_resids,
                loop_resids=loop_resids,
                thick_bb=False,
                show_bio=True,
                show_pharm=pharm_color_dict[pharm_class],
                color_palette=conf_color_dict[loop_name],
                sup_resids=sup_resids,
                sup_coord_path=sup_core_path,
                sup_chainid=sup_chainid,
                set_view=set_view,
                show_resids=show_resids,
            )

    for loop_name, loop_resids in loop_resid_dict.items():

        write_pymol_script(
            interf_df,
            get_file_path(
                f"{loop_name}_{pymol_pml_file}", dir_str=dimer_name, dir_path=out_path
            ),
            loop_resids=loop_resids,
            group_col=loop_name,
            color_palette=conf_color_dict[loop_name],
            style_ribbon=True,
            thick_bb=False,
            show_bio=True,
            show_prot=True,
            coord_path_col=interf_path_col,
            prot_chainid_col=bound_interf_chainid_col,
            sup_resids=sup_resids,
            sup_coord_path=sup_core_path,
            sup_chainid=sup_chainid,
            set_view=mono_view,
            show_resids=show_resids,
        )


def plot_dist(df, dih_dict, sup_core_path, out_path):

    # dist_table_path = get_file_path(
    #     f"{y32_name}_{y71_name}_{dist_table_file}", dir_path=out_path
    # )

    # build_dist_table(
    #         df,
    #         x_resids=[32, 71],
    #         y_resids=[12, 9],
    #         x_atomids=['OH','OH'],
    #         y_atomids=['CA','CA'],
    #         atom_dist_col_lst=[y32_name, y71_name],
    #         dist_table_path=dist_table_path
    #     )

    # dist_df = load_table(dist_table_path)

    # make_facet_plot(
    #     dist_df,
    #     get_file_path(f"{y32_name}_{plot_img_file}", dir_str=y32_name, dir_path=out_path,pre_str=False),
    #     x_col=y32_name,
    #     x_str="Y32(OH):G12(CA)\nDistance (Å)",
    #     plot_width=1.5,
    #     plot_height=1.5,
    #     plot_kde=True,
    #     kde_bw=0.3,
    #     show_legend=False,
    #     v_lines=[10.5],
    #     x_round=0,
    #     y_round=1,
    #     x_lim=[0,20],
    #     x_ticks=[0,5,10,15,20],
    #     hue_col=nuc_class_col,
    #     hue_palette=nuc_color_dict
    # )

    # make_facet_plot(
    #     dist_df,
    #     get_file_path(f"{y71_name}_{plot_img_file}", dir_str=y71_name, dir_path=out_path,pre_str=False),
    #     x_col=y71_name,
    #     x_str="Y71(OH):V9(CA)\nDistance (Å)",
    #     plot_width=1.5,
    #     plot_height=1.5,
    #     plot_kde=True,
    #     kde_bw=0.3,
    #     show_legend=False,
    #     v_lines=[8.75],
    #     x_round=0,
    #     y_round=1,
    #     x_lim=[0,20],
    #     x_ticks=[0,5,10,15,20],
    #     hue_col=nuc_class_col,
    #     hue_palette=nuc_color_dict
    # )

    gtp_df = mask_equal(df, nuc_class_col, gtp_name)

    dist_table_path = get_file_path(
        dist_table_file, dir_str=mut_class_col, dir_path=out_path
    )

    # build_dist_table(
    #     gtp_df,
    #     x_resids=[32],
    #     y_resids=[bio_lig_col],
    #     x_atomids=["OH"],
    #     y_atomids=[gtp_atomids],
    #     atom_dist_col_lst=[atom_dist_col],
    #     hb_status_col_lst=[hb_status_col],
    #     outlier_col_lst=[outlier_col],
    #     dist_table_path=dist_table_path,
    #     check_hb=True,
    # )

    dist_df = load_table(dist_table_path)

    print('All',dist_df[hb_status_col].value_counts())

    make_facet_plot(
        dist_df,
        get_file_path(plot_img_file, dir_str=mut_class_col, dir_path=out_path),
        x_col=atom_dist_col,
        x_str="Y32(OH):3P(O1G)\nDistance (Å)",
        plot_width=1.5,
        plot_height=1.5,
        plot_kde=True,
        show_legend=False,
        x_round=1,
        y_round=1,
        x_lim=[0,15],
        x_ticks=[0.0, 2.5, 5.0, 7.5, 10.0,12.5,15.0],
        hue_col=hb_status_col,
        hue_palette=hb_color_dict,
    )

    # hras_df = mask_equal(
    #     mask_equal(mask_equal(dist_df, bio_lig_col, "GNP"), outlier_col, "False"),
    #     gene_class_col,
    #     hras_name,
    # )
    # write_pymol_script(
    #     hras_df,
    #     get_file_path(
    #         f"{hras_name}_{pymol_pml_file}", dir_str=y32_name, dir_path=out_path
    #     ),
    #     stick_resids=[32],
    #     group_col=hb_status_col,
    #     color_palette=hb_color_dict,
    #     thick_bb=False,
    #     show_bio=True,
    #     sup_coord_path=sup_core_path,
    #     sup_chainid=sup_chainid,
    #     set_view=y32_view,
    #     x_hb_resids=[32],
    #     x_hb_atomids=["OH"],
    #     y_hb_resids=[bio_lig_col],
    #     y_hb_atomids=[gtp_atomids],
    #     show_hb=[sw1_gtp_dir_color],
    #     show_wmhb=[sw1_gtp_wat_color],
    #     show_resids=show_resids,
    # )

    on_df = mask_equal(dist_df, sw1_name, sw1_gtp_in_on_name)
    on_df = mask_equal(on_df, sw2_name, sw2_gtp_in_r_name)

    # dih_table_path = get_file_path(
    #     f"{q61_name}_{dih_table_file}",
    #     dir_str=y32_name,
    #     dir_path=out_path,
    # )

    # build_dih_table(
    #     on_df, dih_dict, chi1_resids=61, chi2_resids=61, dih_table_path=dih_table_path
    # )

    #dih_df = load_table(dih_table_path)

    for mut_class in ["WT", "G12D", "G12V"]:

        mut_df = mask_equal(
            on_df,
            mut_class_col,
            mut_class,
        )

        # make_facet_plot(
        #     mut_df,
        #     get_file_path(
        #         f"{mut_class}_{q61_name}_{chi1_col}_{chi2_col}_{plot_img_file}",
        #         dir_str=y32_name,
        #         dir_path=out_path,
        #     ),
        #     x_col=chi1_col,
        #     y_col=chi2_col,
        #     hue_col=hb_status_col,
        #     hue_palette=hb_color_dict,
        #     x_str=f"χ$^1$",
        #     y_str=f"χ$^2$",
        #     plot_width=1.5,
        #     plot_height=1.5,
        #     kde_bw=1,
        #     show_legend=False,
        # )

        print(mut_class,mut_df[hb_status_col].value_counts())

        write_pymol_script(
            mut_df,
            get_file_path(
                f"{mut_class}_{pymol_pml_file}", dir_str=mut_class_col, dir_path=out_path, pre_str=False
            ),
            stick_resids=[12, 32, 61],
            group_col=hb_status_col,
            color_palette=hb_color_dict,
            thick_bb=False,
            show_bio=True,
            sup_coord_path=sup_core_path,
            sup_chainid=sup_chainid,
            set_view=mut_view,
            x_hb_resids=[12, 12, 32, 61],
            x_hb_atomids=["OD1", "OD1", "OH", "OE1"],
            y_hb_resids=[32, bio_lig_col, bio_lig_col, 32],
            y_hb_atomids=["OH", gtp_atomids, gtp_atomids, "OH"],
            show_hb=[
                hb_color_dict[wmhb_name],
                hb_color_dict[wmhb_name],
                hb_color_dict[hb_name],
                hb_color_dict[hb_name],
            ],
            show_wmhb=[hb_color_dict[wmhb_name]] * 4,
            show_resids=show_resids,
        )


def plot_pockets(df, pocket_df, sup_core_path, out_path):

    make_facet_plot(
        pocket_df,
        get_file_path(
            f"{pocket_volume_col}_{pocket_score_col}_{plot_img_file}",
            dir_str=pocket_class_col,
            dir_path=out_path,
        ),
        x_col=pocket_volume_col,
        y_col=pocket_score_col,
        x_str="Pocket Volume (Å$^3$)",
        y_str="Druggability Score",
        y_round=1,
        col_col=pocket_site_col,
        col_order=pocket_site_lst,
        col_wrap=3,
        hue_col=pocket_type_col,
        hue_palette=pocket_color_dict,
        hue_order=list(pocket_color_dict.keys()),
        show_legend=False,
        plot_width=2.6,
        plot_height=1.2,
        x_rotation=45,
        x_ha="right",
        plot_reg=True,
        log_reg=True,
        trun_reg=True,
        x_lim=[0, 1700],
        x_ticks=[0, 500, 1000, 1500],
        y_lim=[0, 1.2],
        y_ticks=[0.0, 0.5, 1.0],
    )

    for pocket_site in pocket_site_lst:

        if pocket_site == other_pharm_name:
            show_pocket = True
        else:
            show_pocket = pocket_color_dict[pocket_site]

        stick_resids = None
        if pocket_site == sp2_name:
            stick_resids = [12]

        fp_df = mask_equal(pocket_df, pocket_site_col, pocket_site)

        write_pymol_script(
            fp_df,
            get_file_path(
                f"{pocket_site}_{pymol_pml_file}",
                dir_str=pocket_class_col,
                dir_path=out_path,
            ),
            stick_resids=stick_resids,
            loop_resids=[sw1_resids, sw2_resids],
            color_palette=[sw1_color, sw2_color],
            group_col=pocket_type_col,
            color_group=False,
            thick_bb=False,
            sup_resids=sup_resids,
            sup_coord_path=sup_core_path,
            sup_chainid=sup_chainid,
            show_bio=True,
            show_pocket=show_pocket,
            set_view=mono_view,
            coord_path_col=pocket_path_col,
            show_resids=show_resids,
        )

        if pocket_site != other_pharm_name:
            all_df = mask_equal(df, pharm_class_col, pocket_site)

            del fp_df[pocket_path_col]

            fpocket_lst = lst_col(fp_df, pdb_id_col, unique=True)
            all_lst = lst_col(all_df, pdb_id_col, unique=True)

            make_venn_plot(
                fpocket_lst,
                all_lst,
                get_file_path(
                    f"{pocket_site}_{venn_img_file}",
                    dir_str=pocket_class_col,
                    dir_path=out_path,
                ),
                color_1=gray_hex,
                color_2=pocket_color_dict[pocket_site],
                label_1="Predicted",
                label_2="Observed",
                plot_height=1.5,
                plot_width=1.5,
                alpha=0.75,
            )

            write_pymol_script(
                all_df,
                get_file_path(
                    f"{pocket_site}_{y71_name}_{pymol_pml_file}",
                    dir_str=pocket_class_col,
                    dir_path=out_path,
                ),
                stick_resids=[71],
                loop_resids=[71],
                color_palette=resid_color_dict[y71_name],
                group_col=y71_name,
                thick_bb=False,
                sup_resids=sup_resids,
                sup_coord_path=sup_core_path,
                sup_chainid=sup_chainid,
                show_bio=True,
                show_pharm=show_pocket,
                set_view=mono_view,
                show_resids=show_resids,
            )

    pocket_loop_cluster_dict = {
        sp12_name: {
            sw2_name: [
                sw2_gtp_in_r_name,
                sw2_gtp_in_sp12a_name,
                sw2_gtp_in_sp12b_name,
                sw2_gdp_in_sp12_name,
                outlier_name,
                disorder_name
            ]
        },
        sp2_name: {
            sw2_name: [
                sw2_gdp_out_sp2a_name,
                sw2_gdp_out_sp2b_name,
                outlier_name,
                disorder_name
            ]
        },
    }

    pocket_loop_cluster_dict[sp12_name][sw1_name] = [
        sw1_gtp_in_on_name,
        sw1_gdp_out_off_name,
        outlier_name,
        disorder_name
    ]
    pocket_loop_cluster_dict[sp2_name][sw1_name] = [
        sw1_gdp_out_off_name,
        outlier_name,
        disorder_name
    ]

    for pocket_site in [sp2_name, sp12_name]:
        all_df = mask_equal(df, pharm_class_col, pocket_site)
        for loop_name in list(pocket_loop_cluster_dict[pocket_site].keys()):
            color_dict = conf_color_dict[loop_name].copy()

            if loop_name == sw1_name:
                stick_resids = [32]
            elif loop_name == sw2_name:
                stick_resids = [71]

            if pocket_site == sp2_name:
                stick_resids.append(12)

            write_pymol_script(
                all_df,
                get_file_path(
                    f"{loop_name}_{pocket_site}_{pymol_pml_file}",
                    dir_str=pocket_class_col,
                    dir_path=out_path,
                ),
                stick_resids=stick_resids,
                loop_resids=[loop_resid_dict[loop_name]],
                color_palette=color_dict,
                group_col=loop_name,
                color_group=True,
                thick_bb=False,
                sup_resids=sup_resids,
                sup_coord_path=sup_core_path,
                sup_chainid=sup_chainid,
                show_bio=True,
                show_pharm=pocket_color_dict[pocket_site],
                set_view=mono_view,
                coord_path_col=core_path_col,
                show_resids=show_resids,
            )

            make_stacked_barplot(
                plot_df=all_df,
                col_col=match_class_col,
                hue_col=loop_name,
                plot_path=get_file_path(
                    f"{loop_name}_{pocket_site}_{match_class_col}_{plot_img_file}",
                    dir_str=pocket_class_col,
                    dir_path=out_path,
                ),
                hue_palette=color_dict,
                x_str=f"% Structures ({pocket_site} Bound)",
                y_str=f"Inhibitor Chemistry",
                plot_height=1.0,
                plot_width=1.5,
                show_legend=False,
                col_count=True,
                show_barh=True,
            )

            for pocket_status in [pocket_bound_name, pocket_unbound_name]:

                fp_df = mask_equal(
                    mask_equal(
                        pocket_df,
                        pocket_site_col,
                        pocket_site,
                    ),
                    pocket_status_col,
                    pocket_status,
                )

                count_dict = build_col_count_dict(fp_df, loop_name)

                cluster_order = [
                    x
                    for x in list(color_dict.keys())
                    if x in list(count_dict.keys()) and count_dict[x] > 3
                ]

                make_facet_plot(
                    fp_df,
                    get_file_path(
                        f"{loop_name}_{pocket_site}_{pocket_status}_{pocket_volume_col}_{plot_img_file}",
                        dir_str=pocket_class_col,
                        dir_path=out_path,
                        pre_str=False,
                    ),
                    x_col=loop_name,
                    x_order=cluster_order,
                    x_palette=color_dict,
                    x_count=False,
                    y_col=pocket_volume_col,
                    x_str=f"{loop_name} Conformation ({pocket_site}-{pocket_status})",
                    y_str="Pocket Volume (Å$^3$)",
                    show_legend=False,
                    plot_width=1.0,
                    plot_height=1.0,
                    plot_kind="box",
                    marker_size=2,
                    x_pad=35,
                    y_lim=[0, 1700],
                    y_ticks=[0, 500, 1000, 1500],
                )

                make_facet_plot(
                    fp_df,
                    get_file_path(
                        f"{loop_name}_{pocket_site}_{pocket_status}_{pocket_score_col}_{plot_img_file}",
                        dir_str=pocket_class_col,
                        dir_path=out_path,
                        pre_str=False,
                    ),
                    x_col=loop_name,
                    x_order=cluster_order,
                    x_palette=color_dict,
                    x_count=False,
                    y_col=pocket_score_col,
                    x_str=f"{loop_name} Conformation ({pocket_site}-{pocket_status})",
                    y_str="Druggability Score",
                    show_legend=False,
                    plot_width=1.0,
                    plot_height=1.0,
                    plot_kind="box",
                    marker_size=2,
                    y_round=1,
                    x_pad=35,
                    y_lim=[0, 1.2],
                    y_ticks=[0.0, 0.5, 1.0],
                )

    for loop_name in list(loop_resid_dict.keys()):

        color_dict = conf_color_dict[loop_name].copy()

        for pocket_site in [sp2_name, sp12_name, other_pharm_name]:

            if loop_name == sw1_name:
                stick_resids = [32]
            elif loop_name == sw2_name:
                stick_resids = [71]

            if pocket_site == sp2_name:
                stick_resids.append(12)

            show_pocket = True
            if pocket_site != other_pharm_name:
                show_pocket = pocket_color_dict[pocket_site]

            for pocket_status in [pocket_bound_name, pocket_unbound_name]:

                score_cutoff = 0.5

                fp_df = mask_greater(mask_equal(
                    mask_equal(
                        pocket_df,
                        pocket_site_col,
                        pocket_site,
                    ),
                    pocket_status_col,
                    pocket_status,
                ),pocket_score_col,score_cutoff)

                if len(fp_df) > 0:
   
                    write_pymol_script(
                        fp_df,
                        get_file_path(
                            f"{loop_name}_{pocket_site}_{pocket_status}_{score_cutoff}_{pymol_pml_file}",
                            dir_str=f"{pocket_class_col}/{loop_name}",
                            dir_path=out_path,
                            pre_str=False
                        ),
                        stick_resids=stick_resids,
                        loop_resids=[loop_resid_dict[loop_name]],
                        color_palette=color_dict,
                        group_col=loop_name,
                        color_group=True,
                        thick_bb=False,
                        sup_resids=sup_resids,
                        sup_coord_path=sup_core_path,
                        sup_chainid=sup_chainid,
                        show_bio=True,
                        show_pocket=show_pocket,
                        set_view=mono_view,
                        show_resids=show_resids,
                        coord_path_col=pocket_path_col,
                    )



def plot_rascore(build_path, out_path=None, num_cpu=1):

    if out_path is None:
        out_path = f"{os.getcwd()}/{rascore_str}_{plot_str}"

    entry_table_path = get_file_path(entry_table_file, dir_path=build_path)
    pocket_table_path = get_file_path(pocket_table_file, dir_path=build_path)
    interf_table_path = get_file_path(interf_table_file, dir_path=build_path)
    dih_json_path = get_file_path(dih_json_file, dir_path=build_path)

    sup_core_path = get_core_path(sup_pdb_code, sup_chainid, dir_path=build_path)

    df = load_table(entry_table_path)
    pocket_df = load_table(pocket_table_path)
    interf_df = load_table(interf_table_path)
    dih_dict = load_json(dih_json_path)

    sw1_dict = make_dict(lst_col(df, pdb_id_col), lst_col(df, sw1_name))
    sw2_dict = make_dict(lst_col(df, pdb_id_col), lst_col(df, sw2_name))

    pocket_df[sw1_name] = pocket_df[pdb_id_col].map(sw1_dict)
    pocket_df[sw2_name] = pocket_df[pdb_id_col].map(sw2_dict)

    interf_df[sw1_name] = interf_df[pdb_id_col].map(sw1_dict)
    interf_df[sw2_name] = interf_df[pdb_id_col].map(sw2_dict)

    plot_pymol(df, interf_df, sup_core_path, out_path)
    #plot_dist(df, dih_dict, sup_core_path, out_path)
    #plot_pockets(df, pocket_df, sup_core_path, out_path)
    #plot_rama(df, dih_dict, out_path,num_cpu=num_cpu)



# -*- coding: utf-8 -*-
"""
  Copyright 2022 Mitchell Isaac Parker

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

"""

import streamlit as st

from ..constants.pharm import pharm_color_dict, sp2_name
from ..constants.conf import loop_resid_dict, sw1_name, sw2_name, sw1_color, sw2_color
from ..functions.gui import write_st_end, create_st_button, show_st_structure


def home_page():

    style_lst = list()
    surface_lst = list()

    opacity = 1

    surface_lst = [
        [
            {"opacity": opacity, "color": "white"},
            {"chain": "A", "hetflag": False},
        ]
    ]

    style_lst.append(
        [
            {
                "resn": "GDP",
            },
            {
                "stick": {
                    "colorscheme": "whiteCarbon",
                    "radius": 0.2,
                }
            },
        ]
    )

    style_lst.append(
        [
            {
                "resn": "MG",
            },
            {
                "sphere": {
                    "color": "chartreuse",
                    "radius": 0.8,
                }
            },
        ]
    )

    style_lst.append(
        [
            {"chain": "A", "resi": 12, "atom": "CA"},
            {"sphere": {"color": "red", "radius": 0.8}},
        ]
    )

    style_lst.append(
        [
            {
                "chain": "A",
                "resn": "MOV",
                "elem": "C",
            },
            {"stick": {"color": pharm_color_dict[sp2_name], "radius": 0.2}},
        ]
    )
    style_lst.append(
        [
            {
                "chain": "A",
                "resn": "MOV",
                "elem": ["N", "O", "H"],
            },
            {"stick": {"colorscheme": "whiteCarbon", "radius": 0.2}},
        ]
    )

    style_lst.append(
        [
            {
                "chain": "A",
                "resi": 12,
            },
            {"stick": {"colorscheme": "lightgrayCarbon", "radius": 0.2}},
        ]
    )

    for loop_name, loop_resids in loop_resid_dict.items():

        if loop_name == sw1_name:
            loop_color = sw1_color
            stick_resid = 32
        elif loop_name == sw2_name:
            loop_color = sw2_color
            stick_resid = 71

        surface_lst.append(
            [
                {"opacity": opacity, "color": loop_color},
                {"chain": "A", "resi": loop_resids, "hetflag": False},
            ]
        )

        style_lst.append(
            [
                {
                    "chain": "A",
                    "resi": [loop_resids],
                },
                {
                    "cartoon": {
                        "style": "oval",
                        "color": loop_color,
                        "thickness": 0.2,
                    }
                },
            ]
        )

        style_lst.append(
            [
                {
                    "chain": "A",
                    "resi": [stick_resid],
                    "elem": "C",
                },
                {"stick": {"color": loop_color, "radius": 0.2}},
            ]
        )

        style_lst.append(
            [
                {"chain": "A", "resi": [stick_resid], "elem": ["O", "N", "H"]},
                {"stick": {"colorscheme": "whiteCarbon", "radius": 0.2}},
            ]
        )

    left_col, right_col = st.columns(2)

    with left_col:
        show_st_structure(
            "6oim",
            style_lst=style_lst,
            surface_lst=surface_lst,
            cartoon_style="oval",
            spin_on=True,
            zoom_dict={"chain": "A"},
            zoom=1.2,
            width=400,
            height=300,
        )

    right_col.markdown("# Rascore")
    right_col.markdown("### A tool for analyzing RAS protein structures")
    right_col.markdown("**Created by Mitchell Parker and Roland Dunbrack**")
    right_col.markdown("**Fox Chase Cancer Center**")

    database_link_dict = {
        "bioRxiv Paper": "https://www.biorxiv.org/content/10.1101/2022.02.02.478568v1",
        "GitHub Page": "https://github.com/mitch-parker/rascore",
        "RCSB Protein Data Bank": "https://www.rcsb.org",
    }

    st.sidebar.markdown("## Database-Related Links")
    for link_text, link_url in database_link_dict.items():
        create_st_button(link_text, link_url, st_col=st.sidebar)

    community_link_dict = {
        "NCI RAS Initiative": "https://www.cancer.gov/research/key-initiatives/ras",
        "KRAS Kickers": "https://www.kraskickers.org",
        "RASopathies Network": "https://rasopathiesnet.org",
    }

    st.sidebar.markdown("## Community-Related Links")
    for link_text, link_url in community_link_dict.items():
        create_st_button(link_text, link_url, st_col=st.sidebar)

    software_link_dict = {
        "BioPython": "https://biopython.org",
        "RDKit": "https://www.rdkit.org",
        "PDBrenum": "http://dunbrack.fccc.edu/PDBrenum/",
        "Fpocket": "https://bioserv.rpbs.univ-paris-diderot.fr/services/fpocket/",
        "PyMOL": "https://pymol.org/2/",
        "3Dmol": "https://3dmol.csb.pitt.edu",
        "Pandas": "https://pandas.pydata.org",
        "NumPy": "https://numpy.org",
        "SciPy": "https://scipy.org",
        "Sklearn": "https://scikit-learn.org/stable/",
        "Matplotlib": "https://matplotlib.org",
        "Seaborn": "https://seaborn.pydata.org",
        "Streamlit": "https://streamlit.io",
    }

    st.sidebar.markdown("## Software-Related Links")
    link_1_col, link_2_col, link_3_col = st.sidebar.columns(3)

    i = 0
    link_col_dict = {0: link_1_col, 1: link_2_col, 2: link_3_col}
    for link_text, link_url in software_link_dict.items():

        st_col = link_col_dict[i]
        i += 1
        if i == len(link_col_dict.keys()):
            i = 0

        create_st_button(link_text, link_url, st_col=st_col)

    st.markdown("---")

    st.markdown(
        """
        ### Summary
        *Rascore* is a tool for analyzing structures of the RAS protein:
        KRAS, NRAS, and HRAS. The *Rascore* 
        database presents a continually updated analysis of all
        RAS structures in the Protein Data Bank (PDB) with their catalytic SW1 and 
        SW2 loops conformationally classified and their molecular 
        contents annotated (e.g., mutation status, nucleotide state, 
        bound protein, inhibitor site, and others). 

        Details of our work are 
        provided on [bioRxiv](https://www.biorxiv.org/content/10.1101/2022.02.02.478568v1) in the manuscript, 
        **An expanded classification of active, inactive and druggable RAS 
        conformations.** We hope that researchers will use 
        *Rascore* to gain novel insights into RAS biology and 
        drug discovery. 
        """
    )

    st.markdown(
        """
        ### Usage

        To the left, is a main menu for navigating to 
        each page in the *Rascore* database:

        - **Home Page:** We are here!
        - **Database Overview:** Overview of the *Rascore* database, molecular annotations, 
        and conformational classification.
        - **Explore Conformations:** Explore RAS conformations by nucleotide state.
        - **Search PDB:** Search for individual PDB entries containing RAS structures.
        - **Query Database:** Query the *Rascore* database by RAS conformations and molecular annotations.
        - **Compare Inhibitors:** Compare inhibitor-bound RAS structures by individual PDB entries or conformations.
        - **Classify Structures:** Conformationally classify your RAS structures.
        """
    )
    st.markdown("---")

    st.markdown(
        f"""
        ### Authors
        Please feel free to contact us with any issues, comments, or questions.

        ##### Mitchell Parker [![Twitter URL](https://img.shields.io/twitter/url/https/twitter.com/bukotsunikki.svg?style=social&label=Follow%20%40Mitch_P)](https://twitter.com/Mitch_P)

        - Email:  <mip34@drexel.edu> or <mitchell.parker@fccc.edu>
        - GitHub: https://github.com/mitch-parker

        ##### Roland Dunbrack [![Twitter URL](https://img.shields.io/twitter/url/https/twitter.com/bukotsunikki.svg?style=social&label=Follow%20%40RolandDunbrack)](https://twitter.com/RolandDunbrack)

        - Email: <roland.dunbrack@fccc.edu>
        - GitHub: https://github.com/DunbrackLab
        """,
        unsafe_allow_html=True,
    )

    st.markdown("---")

    st.markdown(
        """
        ### Funding

        - NIH F30 GM142263 (to M.P.)
        - NIH R35 GM122517 (to R.D.)
         """
    )

    st.markdown(
        """
        ### License
        Apache License 2.0
        """
    )

    write_st_end()
